"""Howler Agents - Group-Evolving Agents core library."""

from howler_agents.config import HowlerConfig
from howler_agents.evolution.loop import EvolutionLoop

__all__ = ["EvolutionLoop", "HowlerConfig"]
__version__ = "0.1.0"
